@@ -83,33 +83,57 @@ void getDebugLocation(const Region *R, unsigned &LineBegin, unsigned &LineEnd,
       LineEnd = std::max(LineEnd, NewLine);
     }
 }
+}
+
+namespace llvm {
+// @brief Lexicographic order on (line, col) of our debug locations.
+static bool operator<(const llvm::DebugLoc &LHS, const llvm::DebugLoc &RHS) {
+  return LHS.getLine() < RHS.getLine() ||
+         (LHS.getLine() == RHS.getLine() && LHS.getCol() < RHS.getCol());
+}
+}
+
+namespace polly {
+static void getDebugLocations(const Region *R, DebugLoc &Begin, DebugLoc &End) {
+  for (const BasicBlock *BB : R->blocks())
+    for (const Instruction &Inst : *BB) {
+      DebugLoc DL = Inst.getDebugLoc();
+      if (DL.isUnknown())
+        continue;
+
+      Begin = Begin.isUnknown() ? DL : std::min(Begin, DL);
+      End = End.isUnknown() ? DL : std::max(End, DL);
+    }
+}
 
 void emitRejectionRemarks(const llvm::Function &F, const RejectLog &Log) {
   LLVMContext &Ctx = F.getContext();
 
   const Region *R = Log.region();
-  const BasicBlock *Entry = R->getEntry();
-  DebugLoc DL = Entry->getTerminator()->getDebugLoc();
+  DebugLoc Begin, End;
+
+  getDebugLocations(R, Begin, End);
 
   emitOptimizationRemarkMissed(
-      Ctx, DEBUG_TYPE, F, DL,
+      Ctx, DEBUG_TYPE, F, Begin,
       "The following errors keep this region from being a Scop.");
+
   for (RejectReasonPtr RR : Log) {
     const DebugLoc &Loc = RR->getDebugLoc();
     if (!Loc.isUnknown())
       emitOptimizationRemarkMissed(Ctx, DEBUG_TYPE, F, Loc,
                                    RR->getEndUserMessage());
   }
+
+  emitOptimizationRemarkMissed(Ctx, DEBUG_TYPE, F, End,
+                               "Invalid Scop candidate ends here.");
 }
 
 void emitValidRemarks(const llvm::Function &F, const Region *R) {
   LLVMContext &Ctx = F.getContext();
 
-  const BasicBlock *Entry = R->getEntry();
-  const BasicBlock *Exit = R->getExit();
-
-  const DebugLoc &Begin = Entry->getFirstNonPHIOrDbg()->getDebugLoc();
-  const DebugLoc &End = Exit->getFirstNonPHIOrDbg()->getDebugLoc();
+  DebugLoc Begin, End;
+  getDebugLocations(R, Begin, End);
 
   emitOptimizationRemark(Ctx, DEBUG_TYPE, F, Begin,
                          "A valid Scop begins here.");
@@ -125,6 +149,13 @@ const llvm::DebugLoc &RejectReason::getDebugLoc() const {
   return Unknown;
 }
 
+// RejectLog.
+void RejectLog::print(raw_ostream &OS, int level) const {
+  int j = 0;
+  for (auto Reason : ErrorReports)
+    OS.indent(level) << "[" << j++ << "] " << Reason->getMessage() << "\n";
+}
+
 //===----------------------------------------------------------------------===//
 // ReportCFG.
 
@@ -259,6 +290,24 @@ bool ReportVariantBasePtr::classof(const RejectReason *RR) {
   return RR->getKind() == rrkVariantBasePtr;
 }
 
+//===----------------------------------------------------------------------===//
+// ReportDifferentArrayElementSize
+
+std::string ReportDifferentArrayElementSize::getMessage() const {
+  return "Access to one array through data types of different size";
+}
+
+bool ReportDifferentArrayElementSize::classof(const RejectReason *RR) {
+  return RR->getKind() == rrkDifferentElementSize;
+}
+
+std::string ReportDifferentArrayElementSize::getEndUserMessage() const {
+  llvm::StringRef BaseName = BaseValue->getName();
+  std::string Name = (BaseName.size() > 0) ? BaseName : "UNKNOWN";
+  return "The array \"" + Name + "\" is accessed through elements that differ "
+                                 "in size";
+}
+
 //===----------------------------------------------------------------------===//
 // ReportNonAffineAccess.
 
@@ -270,6 +319,12 @@ bool ReportNonAffineAccess::classof(const RejectReason *RR) {
   return RR->getKind() == rrkNonAffineAccess;
 }
 
+std::string ReportNonAffineAccess::getEndUserMessage() const {
+  llvm::StringRef BaseName = BaseValue->getName();
+  std::string Name = (BaseName.size() > 0) ? BaseName : "UNKNOWN";
+  return "The array subscript of \"" + Name + "\" is not affine";
+}
+
 //===----------------------------------------------------------------------===//
 // ReportIndVar.
 
@@ -295,43 +350,6 @@ bool ReportPhiNodeRefInRegion::classof(const RejectReason *RR) {
   return RR->getKind() == rrkPhiNodeRefInRegion;
 }
 
-//===----------------------------------------------------------------------===//
-// ReportNonCanonicalPhiNode.
-
-ReportNonCanonicalPhiNode::ReportNonCanonicalPhiNode(Instruction *Inst)
-    : ReportIndVar(rrkNonCanonicalPhiNode), Inst(Inst) {}
-
-std::string ReportNonCanonicalPhiNode::getMessage() const {
-  return "Non canonical PHI node: " + *Inst;
-}
-
-const DebugLoc &ReportNonCanonicalPhiNode::getDebugLoc() const {
-  return Inst->getDebugLoc();
-}
-
-bool ReportNonCanonicalPhiNode::classof(const RejectReason *RR) {
-  return RR->getKind() == rrkNonCanonicalPhiNode;
-}
-
-//===----------------------------------------------------------------------===//
-// ReportLoopHeader.
-
-ReportLoopHeader::ReportLoopHeader(Loop *L)
-    : ReportIndVar(rrkLoopHeader), L(L) {}
-
-std::string ReportLoopHeader::getMessage() const {
-  return ("No canonical IV at loop header: " + L->getHeader()->getName()).str();
-}
-
-const DebugLoc &ReportLoopHeader::getDebugLoc() const {
-  BasicBlock *BB = L->getHeader();
-  return BB->getTerminator()->getDebugLoc();
-}
-
-bool ReportLoopHeader::classof(const RejectReason *RR) {
-  return RR->getKind() == rrkLoopHeader;
-}
-
 //===----------------------------------------------------------------------===//
 // ReportIndEdge.
 
@@ -356,7 +374,8 @@ bool ReportIndEdge::classof(const RejectReason *RR) {
 // ReportLoopBound.
 
 ReportLoopBound::ReportLoopBound(Loop *L, const SCEV *LoopCount)
-    : RejectReason(rrkLoopBound), L(L), LoopCount(LoopCount) {
+    : RejectReason(rrkLoopBound), L(L), LoopCount(LoopCount),
+      Loc(L->getStartLoc()) {
   ++BadLoopBoundForScop;
 }
 
@@ -365,15 +384,16 @@ std::string ReportLoopBound::getMessage() const {
          L->getHeader()->getName();
 }
 
-const DebugLoc &ReportLoopBound::getDebugLoc() const {
-  const BasicBlock *BB = L->getHeader();
-  return BB->getTerminator()->getDebugLoc();
-}
+const DebugLoc &ReportLoopBound::getDebugLoc() const { return Loc; }
 
 bool ReportLoopBound::classof(const RejectReason *RR) {
   return RR->getKind() == rrkLoopBound;
 }
 
+std::string ReportLoopBound::getEndUserMessage() const {
+  return "Failed to derive an affine function from the loop bounds.";
+}
+
 //===----------------------------------------------------------------------===//
 // ReportFuncCall.
 
@@ -391,7 +411,7 @@ const DebugLoc &ReportFuncCall::getDebugLoc() const {
 }
 
 std::string ReportFuncCall::getEndUserMessage() const {
-  return "This function call cannot be handeled. "
+  return "This function call cannot be handled. "
          "Try to inline it.";
 }
 
@@ -403,27 +423,26 @@ bool ReportFuncCall::classof(const RejectReason *RR) {
 // ReportAlias.
 
 ReportAlias::ReportAlias(Instruction *Inst, AliasSet &AS)
-    : RejectReason(rrkAlias), Inst(Inst), AS(AS) {
-  ++BadAliasForScop;
-}
-
-std::string ReportAlias::formatInvalidAlias(AliasSet &AS) const {
-  std::string Message;
-  raw_string_ostream OS(Message);
-
-  OS << "Possible aliasing: ";
-
-  std::vector<Value *> Pointers;
+    : RejectReason(rrkAlias), Inst(Inst) {
 
   for (const auto &I : AS)
     Pointers.push_back(I.getValue());
 
-  std::sort(Pointers.begin(), Pointers.end());
+  ++BadAliasForScop;
+}
 
-  for (std::vector<Value *>::iterator PI = Pointers.begin(),
-                                      PE = Pointers.end();
+std::string ReportAlias::formatInvalidAlias(std::string Prefix,
+                                            std::string Suffix) const {
+  std::string Message;
+  raw_string_ostream OS(Message);
+
+  OS << Prefix;
+
+  for (PointerSnapshotTy::const_iterator PI = Pointers.begin(),
+                                         PE = Pointers.end();
        ;) {
-    Value *V = *PI;
+    const Value *V = *PI;
+    assert(V && "Diagnostic info does not match found LLVM-IR anymore.");
 
     if (V->getName().size() == 0)
       OS << "\"" << *V << "\"";
@@ -438,10 +457,19 @@ std::string ReportAlias::formatInvalidAlias(AliasSet &AS) const {
       break;
   }
 
+  OS << Suffix;
+
   return OS.str();
 }
 
-std::string ReportAlias::getMessage() const { return formatInvalidAlias(AS); }
+std::string ReportAlias::getMessage() const {
+  return formatInvalidAlias("Possible aliasing: ");
+}
+
+std::string ReportAlias::getEndUserMessage() const {
+  return formatInvalidAlias("Accesses to the arrays ",
+                            " may access the same memory.");
+}
 
 const DebugLoc &ReportAlias::getDebugLoc() const { return Inst->getDebugLoc(); }
 
