@@ -18,7 +18,7 @@ using namespace llvm;
 using namespace polly;
 
 bool ScopPass::runOnRegion(Region *R, RGPassManager &RGM) {
-  S = 0;
+  S = nullptr;
 
   if ((S = getAnalysis<ScopInfo>().getScop()))
     return runOnScop(*S);
