@@ -30,6 +30,7 @@
 #include "polly/LinkAllPasses.h"
 #include "polly/Options.h"
 #include "polly/ScopInfo.h"
+#include "polly/Support/GICHelper.h"
 #include "llvm/Support/Debug.h"
 
 using namespace llvm;
@@ -344,12 +345,12 @@ isl_map *IslScheduleOptimizer::getPrevectorMap(isl_ctx *ctx, int DimToVectorize,
   // DimToVectorize to the point loop at the innermost dimension.
   for (int i = 0; i < ScheduleDimensions; i++) {
     c = isl_equality_alloc(isl_local_space_copy(LocalSpace));
-    isl_constraint_set_coefficient_si(c, isl_dim_in, i, -1);
+    c = isl_constraint_set_coefficient_si(c, isl_dim_in, i, -1);
 
     if (i == DimToVectorize)
-      isl_constraint_set_coefficient_si(c, isl_dim_out, PointDimension, 1);
+      c = isl_constraint_set_coefficient_si(c, isl_dim_out, PointDimension, 1);
     else
-      isl_constraint_set_coefficient_si(c, isl_dim_out, i, 1);
+      c = isl_constraint_set_coefficient_si(c, isl_dim_out, i, 1);
 
     TilingMap = isl_map_add_constraint(TilingMap, c);
   }
@@ -501,11 +502,9 @@ bool IslScheduleOptimizer::runOnScop(Scop &S) {
   }
 
   DEBUG(dbgs() << "\n\nCompute schedule from: ");
-  DEBUG(dbgs() << "Domain := "; isl_union_set_dump(Domain); dbgs() << ";\n");
-  DEBUG(dbgs() << "Proximity := "; isl_union_map_dump(Proximity);
-        dbgs() << ";\n");
-  DEBUG(dbgs() << "Validity := "; isl_union_map_dump(Validity);
-        dbgs() << ";\n");
+  DEBUG(dbgs() << "Domain := " << stringFromIslObj(Domain) << ";\n");
+  DEBUG(dbgs() << "Proximity := " << stringFromIslObj(Proximity) << ";\n");
+  DEBUG(dbgs() << "Validity := " << stringFromIslObj(Validity) << ";\n");
 
   int IslFusionStrategy;
 
@@ -555,7 +554,7 @@ bool IslScheduleOptimizer::runOnScop(Scop &S) {
   if (!Schedule)
     return false;
 
-  DEBUG(dbgs() << "Schedule := "; isl_schedule_dump(Schedule); dbgs() << ";\n");
+  DEBUG(dbgs() << "Schedule := " << stringFromIslObj(Schedule) << ";\n");
 
   isl_union_map *ScheduleMap = getScheduleMap(Schedule);
 
