@@ -1,46 +1,57 @@
 #!/bin/sh
 
-CLOOG_HASH="22643c94eba7b010ae4401c347289f4f52b9cd2b"
-ISL_HASH="163646566efd07086ed4fbe99ecbf8d5bb8ebca0"
+ISL_HASH="0698f8436c523ecc742b13a9b3aa337cc2421fa2"
 
 PWD=`pwd`
 
 check_command_line() {
   if [ $# -eq 1 ]
   then
-    CLOOG_DIR="${1}"
+    ISL_DIR="${1}"
   else
-      echo "Usage: " ${0} '<Directory to checkout CLooG>'
+      echo "Usage: " ${0} '<Directory to checkout isl>'
       exit 1
   fi
 }
 
-check_cloog_directory() {
-  if ! [ -e ${CLOOG_DIR} ]
+check_isl_directory() {
+  if ! [ -e ${ISL_DIR} ]
   then
-    echo :: Directory "'${CLOOG_DIR}'" does not exists. Trying to create it.
-    if ! mkdir -p "${CLOOG_DIR}"
+    echo :: Directory "'${ISL_DIR}'" does not exists. Trying to create it.
+    if ! mkdir -p "${ISL_DIR}"
     then exit 1
     fi
   fi
 
-  if ! [ -d ${CLOOG_DIR} ]
+  if ! [ -d ${ISL_DIR} ]
   then
-    echo "'${CLOOG_DIR}'" is not a directory
+    echo "'${ISL_DIR}'" is not a directory
     exit 1
   fi
 
   # Make it absolute
-  cd ${CLOOG_DIR}
-  CLOOG_DIR=`pwd`
+  cd ${ISL_DIR}
+  ISL_DIR=`pwd`
 
-  if ! [ -e "${CLOOG_DIR}/.git" ]
+  if ! [ -e "${ISL_DIR}/.git" ]
   then
     echo ":: No git checkout found"
     IS_GIT=0
   else
     echo ":: Existing git repo found"
-    IS_GIT=1
+
+    git log cc726006058136865f8c2f496d3df57b9f937ea5 2> /dev/null > /dev/null
+    OUT=$?
+    if [ $OUT -eq 0 ];then
+         echo ":: ISL repository found!"
+         IS_GIT=1
+    else
+         echo ":: Unknown repository found (CLooG?)!"
+         echo ":: Moving it to ${ISL_DIR}_old"
+         run mv ${ISL_DIR} ${ISL_DIR}_old
+         run mkdir ${ISL_DIR}
+         IS_GIT=0
+    fi
   fi
 }
 
@@ -58,39 +69,29 @@ run() {
 }
 
 check_command_line $@
-check_cloog_directory
-
-ISL_DIR=${CLOOG_DIR}/isl
+check_isl_directory
 
 if [ ${IS_GIT} -eq 0 ]
 then
   echo :: Performing initial checkout
   # Remove the existing CLooG and ISL dirs to avoid crashing older git versions.
-  cd ${CLOOG_DIR}/..
-  run rmdir "${CLOOG_DIR}"
-  run git clone http://repo.or.cz/r/cloog.git ${CLOOG_DIR}
+  cd ${ISL_DIR}/..
   run rmdir "${ISL_DIR}"
   run git clone http://repo.or.cz/r/isl.git ${ISL_DIR}
 fi
 
-echo :: Fetch versions required by Polly
-run cd ${CLOOG_DIR}
+echo :: Fetch version required by Polly
+run cd ${ISL_DIR}
 run git remote update
-run cd isl
-run git remote update
-
-echo :: Setting CLooG version
-run cd ${CLOOG_DIR}
-run git reset --hard "${CLOOG_HASH}"
 
 echo :: Setting isl version
 run cd ${ISL_DIR}
 run git reset --hard "${ISL_HASH}"
 
 echo :: Generating configure
-run cd ${CLOOG_DIR}
+run cd ${ISL_DIR}
 run ./autogen.sh
 
-echo :: If you install cloog/isl the first time run "'./configure'" followed by
+echo :: If you install isl the first time run "'./configure'" followed by
 echo :: "'make'" and "'make install'", otherwise, just call "'make'" and
 echo :: "'make'" install.
